" vim: noexpandtab ts=4 sw=4

if exists("b:current_syntax")
	finish
endif

" Lexer-defined rules
syn match		maddyComment	"#.*"
syn region      maddyString		start=+"+ skip=+\\\\\|\\"+ end=+"+ oneline

syn region		maddyBlock		start="{" end="}" transparent fold

hi def link maddyComment	Comment
hi def link maddyString		String

" Parser-defined rules
syn match		maddyMacroName	"[a-z0-9_]" contained containedin=maddyMacro
syn match		maddyMacro      "$(.\{-})" contains=maddyMacroName

syn match		maddyMacroDefSign "=" contained
syn match		maddyMacroDef     "\^$([a-z0-9_]\{-})\s=\s.\+" contains=maddyMacro,maddyMacroDefSign

hi def link maddyMacroName			Identifier
hi def link maddyMacro				Special
hi def link maddyMacroDefSign		Special

" config.Map values
syn keyword maddyBool yes no

syn match maddyInt '\<\d\+\>'
syn match maddyInt '\<[-+]\d\+\>'
syn match maddyFloat '\<[-+]\d\+\.\d*\<'

syn match maddyReference /[ \t]&[^ \t]\+/ms=s+1 contains=maddyReferenceSign
syn match maddyReferenceSign /&/ contained

hi def link maddyBool		Boolean
hi def link maddyInt		Number
hi def link maddyFloat		Float

hi def link maddyReferenceSign	Special

" Module values

" grep --no-file -E 'Register.*\(".+", ' **.go | sed -E 's/.+Register.*\("([^"]+)", .+/\1/' | sort -u
syn keyword maddyModule
	\ dnsbl
	\ dummy
	\ extauth
	\ imap
	\ lmtp
	\ pam
	\ queue
	\ remote
	\ replace_rcpt
	\ replace_sender
	\ require_matching_ehlo
	\ require_matching_rdns
	\ require_mx_record
	\ shadow
	\ sign_dkim
	\ smtp
	\ smtp_downstream
	\ sql
	\ submission
	\ test_check
	\ test_modifier
	\ verify_dkim

syn keyword maddyDispatchDir
	\ check
	\ modify
	\ default_source
	\ source
	\ default_destination
	\ destination
	\ reject
	\ deliver_to
	\ reroute
	\ dmarc

" grep --no-file -E 'cfg..+\(".+", ' **.go | sed -E 's/.+cfg..+\("([^"]+)", .+/\1/' | sort -u
syn keyword maddyModDir
	\ allow_body_subset
	\ allow_multiple_from
	\ appendlimit
	\ attempt_starttls
	\ auth
	\ autogenerated_msg_domain
	\ body_canon
	\ bounce
	\ broken_sig_action
	\ check_early
	\ client_ipv4
	\ client_ipv6
	\ compression
	\ concurrency
	\ debug
	\ defer_sender_reject
	\ domain
	\ domains
	\ driver
	\ dsn
	\ ehlo
	\ enforce_early
	\ fail_action
	\ fail_open
	\ files
	\ fsstore
	\ hash
	\ header_canon
	\ helper
	\ hostname
	\ insecure_auth
	\ io_debug
	\ io_errors
	\ junk_mailbox
	\ key_path
	\ location
	\ mailfrom
	\ max_logged_rcpt_errors
	\ max_message_size
	\ max_parallelism
	\ max_recipients
	\ max_tries
	\ min_mx_level
	\ min_tls_level
	\ mtasts_cache
	\ neutral_action
	\ newkey_algo
	\ none_action
	\ no_sig_action
	\ oversign_fields
	\ perdomain
	\ permerr_action
	\ quarantine_threadhold
	\ ratelimit
	\ read_timeout
	\ reject_threadhold
	\ required_fields
	\ require_sender_match
	\ require_tls
	\ responses
	\ run_on
	\ score
	\ selector
	\ sig_expiry
	\ sign_fields
	\ softfail_action
	\ sqlite3_busy_timeout
	\ sqlite3_cache_size
	\ sqlite3_exclusive_lock
	\ storage
	\ target
	\ targets
	\ temperr_action
	\ tls
	\ tls_client
	\ use_helper
	\ write_timeout

hi def link maddyModDir		Identifier
hi def link maddyModule		Identifier
hi def link maddyDispatchDir		Identifier

let b:current_syntax = "maddy"
